import numpy as np
from scipy.sparse.linalg import cg, LinearOperator
from tqdm import tqdm
import sigpy as sp

def admm_l1(b, Afun, Atfun, lam, rho, num_iters, imageResolution):
    
    # Wavelet transform operator
    W = sp.linop.Wavelet(imageResolution)
    
    def Wfun(x):
        return W * x

    def Wtfun(x):
        return W.H * x

    # initialize x,z,u with all zeros
    x = np.zeros(imageResolution, dtype=np.complex64)
    z = np.zeros(Wfun(x).shape, dtype=np.complex64)
    u = np.zeros(Wfun(x).shape, dtype=np.complex64)    

    for it in tqdm(range(num_iters)):

        # x update using cg solver
        v = z-u        

        cg_iters = 50           # number of iterations for CG solver
        cg_tolerance = 1e-12    # convergence tolerance of cg solver

        size = (imageResolution[0]*imageResolution[1], imageResolution[0]*imageResolution[1])
        method = lambda x: Atfun(Afun(x.reshape(imageResolution))) + rho * Wtfun(Wfun(x.reshape(imageResolution)))
        A_tilt = LinearOperator(size, matvec=method)
        b_tilt = (Atfun(b) + rho * Wtfun(z - u)).reshape((imageResolution[0]*imageResolution[1],))
        x_ln = cg(A_tilt, b_tilt, maxiter=cg_iters, tol=cg_tolerance)[0]
        x = x_ln.reshape(imageResolution)

        # z update - soft shrinkage    
        kappa = lam / rho
        v = Wfun(x) + u        
        
        z = np.maximum(1 - kappa/np.abs(v), 0) * v
        
        # u-update
        u = u + Wfun(x) - z

    return x
