import numpy as np
from scipy.sparse.linalg import cg, LinearOperator
from tqdm import tqdm

from finite_differences import *

def admm_tv(b, Afun, Atfun, lam, rho, num_iters, imageResolution, anisotropic_tv=True):

    # initialize x,z,u with all zeros
    x = np.zeros(imageResolution, dtype=np.complex64)
    z = np.zeros((2, imageResolution[0], imageResolution[1]), dtype=np.complex64)
    u = np.zeros((2, imageResolution[0], imageResolution[1]), dtype=np.complex64)    

    for it in tqdm(range(num_iters)):

        # x update using cg solver
        v = z-u        

        cg_iters = 50           # number of iterations for CG solver
        cg_tolerance = 1e-12    # convergence tolerance of cg solver

        size = (imageResolution[0]*imageResolution[1], imageResolution[0]*imageResolution[1])
        method = lambda x: Atfun(Afun(x.reshape(imageResolution))) + rho * opDtx(opDx(x.reshape(imageResolution)))
        A_tilt = LinearOperator(size, matvec=method)
        b_tilt = (Atfun(b) + rho * opDtx(z-u)).reshape((imageResolution[0]*imageResolution[1],))
        x_ln = cg(A_tilt, b_tilt, maxiter=cg_iters, tol=cg_tolerance)[0]
        x = x_ln.reshape(imageResolution) 

        # z update - soft shrinkage    
        kappa = lam / rho
        v = opDx(x) + u        

        # proximal operator of anisotropic TV term
        if anisotropic_tv:
            z = np.maximum(1 - kappa/np.abs(v), 0) * v
        
        # proximal operator of isotropic TV term
        else:
            vnorm = np.sqrt(np.abs(v[0,:,:])**2 + np.abs(v[1,:,:])**2)
            z[0,:,:] = np.maximum(1 - kappa/vnorm,0) * v[0,:,:]
            z[1,:,:] = np.maximum(1 - kappa/vnorm,0) * v[1,:,:]        

        # u-update
        u = u + opDx(x) - z

    return x
