# import packages
import skimage.io as io
import numpy as np
from numpy.fft import fft2, ifft2, fftshift, ifftshift
import sigpy as sp

def retro_undersample(im_xy, us_rate):
    nx, ny = im_xy.shape
    sample_mask = np.zeros((nx*ny,))
    sample_mask[:np.int64(us_rate*nx*ny)] = 1
    np.random.shuffle(sample_mask)
    sample_mask = np.reshape(sample_mask,(nx,ny))
    
    ksp_xy = sp.fft(im_xy)
    ksp_us_xy = ksp_xy * sample_mask

    im_us_xy = np.abs(sp.ifft(ksp_us_xy))
    im_us_xy = im_us_xy / np.max(im_us_xy)
    return im_us_xy, sample_mask

# linear operator in data consistency term
def Afun(x):
    # input x = im_xy shape (nx, ny), output k-space data shape (nx*ny, )
    ksp_xy = sp.fft(x)
    ksp_us = np.reshape(ksp_xy * sample_mask, (nx*ny,))
    return ksp_us
 
def Atfun(x):
    # input x = ksp_us shape (nx*ny,), output image shape (nx, ny)
    ksp_us_xy = np.reshape(x, (nx, ny))
    im_us_xy = sp.ifft(ksp_us_xy)
    return im_us_xy
