import numpy as np
from PIL import Image
import torch.utils.data as data
import scipy.io as sio
import random
import os
import h5py

from util.imageUtil import *


def getDataloader(dataType = '',mode = 'train', batchSize = 1,  us_rate=0.4, is_train=True, imgpath='', cal=None):
    if(mode == 'train'):
        shuffleFlag = True
    else:
        shuffleFlag = False


    if('complex' in dataType):
        dataMode = 'complex'
    elif('abs' in dataType):
        dataMode = 'abs'
    else:
        dataMode = 'abs'


    if('total_random' in dataType):
        if cal is not None:
            samplingMode = 'total_random_cal'
        else:
            samplingMode = 'total_random'
    else:
        assert False,"samplingMode ERROR: No samplingMode Include"


    dataset = Get_Img(imgpath, dataMode, samplingMode, us_rate=us_rate, is_train=is_train, cal=cal)

    data_loader = data.DataLoader(dataset, batch_size=batchSize, shuffle=shuffleFlag)
    datasize = len(dataset)
    return data_loader,datasize


class Get_Img(data.Dataset):
    def __init__(self, imgpath, mode='abs', samplingMode='default', us_rate=0.4, is_train=True, cal=None):
        self.yList = []
        self.mList = []
        self.train = is_train
        assert mode in ['complex', 'abs'], "real mode is abandoned"
        self.mode = mode
        self.us_rate = us_rate
        if cal is not None:
            self.cal = int(cal)
        else:
            self.cal = cal

        index = 0

        self.imagenames = os.listdir(imgpath)
        #self.imagenames = [i for i in self.imagenames if 'jpg' in i]
        for f in self.imagenames:
            filename = imgpath + f
            im = Image.open(filename)
            im = im.resize((256, 256), Image.BILINEAR)
            #im.rezie()
            im_np = np.array(im).astype(np.float32) / 255.
            if len(im_np.shape) == 3:
                im_np = im_np[:, :, 0]
            if samplingMode == 'total_random':
                if not is_train:
                    randI = index
                    random.seed(randI)
                    pickindex = random.sample(np.arange(0, im_np.shape[0] * im_np.shape[1]).tolist(), np.int64(us_rate * im_np.shape[0] * im_np.shape[1]))
                    mask = np.zeros((im_np.shape[0] * im_np.shape[1],))
                    mask[pickindex] = 1
                    mask = np.reshape(mask, (im_np.shape[-2], im_np.shape[-1]))

            elif samplingMode == 'total_random_cal':
                if not is_train:
                    randI = index
                    random.seed(randI)
                    pickindex = random.sample(np.arange(0, im_np.shape[0] * im_np.shape[1]).tolist(), np.int64(us_rate * im_np.shape[0] * im_np.shape[1]))
                    mask = np.zeros((im_np.shape[0] * im_np.shape[1],))
                    mask[pickindex] = 1
                    mask = np.reshape(mask, (im_np.shape[-2], im_np.shape[-1]))
                    mask[im_np.shape[0] // 2 - self.cal:im_np.shape[0] // 2 + self.cal, im_np.shape[1] // 2 - self.cal:im_np.shape[1] // 2 + self.cal] = 1
            if not is_train:
                m = np.fft.ifftshift(mask)
            else:
                m = [-1]

            y = np.zeros((1, 256, 256))
            if (mode == 'abs'):
                y = np.zeros((1, 256, 256))
            elif (mode == 'complex'):
                y = np.zeros((2, 256, 256))
            else:
                assert False, "real mode is abandoned"
            y[0] = im_np

            self.yList.append(y)
            self.mList.append(m)
            index += 1

    def __getitem__(self, index):
        i = index
        img = self.mode
        label = self.yList[i]
        if self.train:
            pickindex = random.sample(np.arange(0, label.shape[-2] * label.shape[-1]).tolist(),
                                      np.int64(self.us_rate * label.shape[-2] * label.shape[-1]))
            mask = np.zeros((label.shape[-2] * label.shape[-1],))
            mask[pickindex] = 1
            mask = np.reshape(mask, (label.shape[-2], label.shape[-1]))
            if self.cal is not None:
                mask[label.shape[-2] // 2 - self.cal:label.shape[-2] // 2 + self.cal, label.shape[-1] // 2 - self.cal:label.shape[-1] // 2 + self.cal] = 1
            mask = np.fft.ifftshift(mask)
        else:
            mask = self.mList[i]

        return img, label, mask

    def __len__(self):
        return len(self.yList)


