# +
import run as run
import numpy as np
import sys
import matplotlib.pyplot as plt
from PIL import Image

from admm_utils import *
from admm_l1 import *
from admm_tv import *
import sigpy as sp

import argparse


parser = argparse.ArgumentParser(description="Single image test")
parser.add_argument('--mode', type=str, default='ADMM_TVa', help='method for reconstruction. Please choose one from l1_wavelet, ADMM_TVa, ADMM_TVi and CRNN')
parser.add_argument('--us', type=float, default='0.4', help='undersampling rate')
parser.add_argument('--imgpath', type=str, default='', help='image path')
parser.add_argument('--savepath', type=str, default='', help='path for saving reconstruction image')
parser.add_argument('--model_load_path',  type=str, default='', help='model checkpoint (only for testing CRNN)')
parser.add_argument('--configName', type=str, default='default', help='CRNN model config')
args = parser.parse_args()

# -

def test_img(mode, us_r, imagename, savepath, load_path='', configName='', num_iters=30, rho=3e-2, lam=3e-3):
    im = Image.open(imagename)
    im = im.resize((256, 256), Image.BILINEAR)
    img = np.array(im).astype(np.float32) / 255.
    if len(img.shape) == 3:
        img = img[:, :, 0]
    
    nx, ny = img.shape
    
    if mode == 'CRNN':
        #configName = 'default'
        filename = 'config/' + configName + '.ini'
        c = run.core(filename, single_image_test=True)
        results = c.single_validation('complex', imagename, load_path, us_r)
        plt.imsave(savepath + 'CRNN_' + str(us_r) + '.png', results['rec_2'][0, :, :][:, :, None].repeat(3, 2))
    else:
        im_us_xy, sample_mask = retro_undersample(img, us_r)
        
        def Afun(x):
            # input x = im_xy shape (nx, ny)
            ksp_xy = sp.fft(x)
            ksp_us = np.reshape(ksp_xy * sample_mask, (nx*ny,))
            return ksp_us
 
        def Atfun(x):
            # input x = ksp_us shape (nx*ny,)
            ksp_us_xy = np.reshape(x, (nx, ny))
            im_us_xy = sp.ifft(ksp_us_xy)
            return im_us_xy
        
        if mode == 'l1_wavelet':
            b = Afun(img)
            im_admm_l1 = admm_l1(b, Afun, Atfun, lam, rho, num_iters, img.shape)
            im_admm_l1 = np.abs(im_admm_l1)
            plt.imsave(savepath + 'l1_wavelet_' + str(us_r) + '.png', im_admm_l1, cmap='gray')
        elif mode == 'ADMM_TVa':
            b = Afun(img)
            im_admm_tva = admm_tv(b, Afun, Atfun, lam, rho, num_iters, img.shape, anisotropic_tv=True)
            im_admm_tva = np.abs(im_admm_tva)
            plt.imsave(savepath + 'ADMM_TVa' + str(us_r) + '.png', im_admm_tva, cmap='gray')
        elif mode == 'ADMM_TVi':
            b = Afun(img)
            im_admm_tvi = admm_tv(b, Afun, Atfun, lam, rho, num_iters, img.shape, anisotropic_tv=False)
            im_admm_tvi = np.abs(im_admm_tvi)
            plt.imsave(savepath + 'ADMM_TVi' + str(us_r) + '.png', im_admm_tvi,cmap='gray')
        else:
            assert 'mode type wrong!'


if __name__ == '__main__':
	mode = args.mode
	#### undersampling rate
	us_r = args.us
	savepath = args.savepath#'./img_results/'
	#### image path
	imagename = args.imgpath#'./data/Knee_data/vali/1158_11.jpg'
	#### for CRNN (checkpoint)
	load_path = args.model_load_path#'./result/total_random_sampling_rate_08_KNEE/CN_dOri_c5_complex_tr_trick2/weight/saved_500.pkl'
	#### for ADMM
	num_iters=30 
	rho=3e-2 # empirically found optimal value for TV priors, 1e-2 for l1 wavelet
	lam=3e-3 # refer to report for optimal values 
	
	#### single image test
	test_img(mode, us_r, imagename, savepath,  load_path, configName=args.configName)
	# test_img(mode, us_r, imagename, savepath,  load_path, configName=args.configName, num_iters, rho, lam)
